package digital.laboratory.platform.reagent.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.reagent.dto.ComplianceCheckDTO;
import digital.laboratory.platform.reagent.dto.OutgoingContentsDTO;
import digital.laboratory.platform.reagent.entity.*;
import digital.laboratory.platform.reagent.mapper.DeliveryRegistrationFormMapper;
import digital.laboratory.platform.reagent.service.*;
import digital.laboratory.platform.reagent.vo.DeliveryRegistrationFormVO;
import digital.laboratory.platform.reagent.vo.OutgoingContentsVO;
import digital.laboratory.platform.reagent.vo.ReagentConsumablesVO;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

/**
 * (试剂耗材出库登记表)服务实现类
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (试剂耗材出库登记表) 服务实现类
 */
@Service
@SuppressWarnings("all")
public class DeliveryRegistrationFormServiceImpl extends ServiceImpl<DeliveryRegistrationFormMapper, DeliveryRegistrationForm> implements DeliveryRegistrationFormService {

    @Autowired
    private ApplicationForUseService applicationForUseService;

    @Autowired
    private ReagentConsumablesSetService reagentConsumablesSetService;

    @Autowired
    private ReagentConsumablesService reagentConsumablesService;

    @Autowired
    private OutgoingContentsService outgoingContentsService;

    @Autowired
    private DeliveryRegistrationFormService deliveryRegistrationFormService;

    @Autowired
    private ComplianceCheckService complianceCheckService;

    @Autowired
    private StandardMaterialApplicationService standardMaterialApplicationService;

    @Autowired
    private RequisitionRecordService requisitionRecordService;

    @Autowired
    private ReagentConsumableInventoryService reagentConsumableInventoryService;

    @Autowired
    private SolutionUseFormService solutionUseFormService;

    @Autowired
    private ReferenceMaterialService referenceMaterialService;

    @Override
    @Transactional//创建出库任务
    public DeliveryRegistrationForm addFrom(String applicationForUseId) {

        ApplicationForUse byId = applicationForUseService.getById(applicationForUseId);
        //创建出库登记表
        DeliveryRegistrationForm deliveryRegistrationForm = new DeliveryRegistrationForm();

        deliveryRegistrationForm.setId(IdWorker.get32UUID().toUpperCase());
        deliveryRegistrationForm.setOutgoingApplicantId(byId.getRecipientId());
        deliveryRegistrationForm.setStatus(0);
        byId.setDeliveryRegistrationFormId(deliveryRegistrationForm.getId());

        LambdaQueryWrapper<ReagentConsumablesSet> reagentConsumablesSetLambdaQueryWrapper = new LambdaQueryWrapper<>();
        //得到领用集合
        reagentConsumablesSetLambdaQueryWrapper.eq(ReagentConsumablesSet::getApplicationForUseId, byId.getId());

        List<ReagentConsumablesSet> list = reagentConsumablesSetService.list(reagentConsumablesSetLambdaQueryWrapper);

        List<OutgoingContents> outgoingContentsList = new ArrayList<>();
        //循环领用集合，创建出库集合
        for (ReagentConsumablesSet reagentConsumablesSet : list) {

            OutgoingContents outgoingContents = new OutgoingContents();

            ReagentConsumables one = reagentConsumablesService.getById(reagentConsumablesSet.getReagentConsumableId());

            if (one.getCategory().equals("标准物质")|one.getCategory().equals("标准储备溶液")){

                LambdaQueryWrapper<ReferenceMaterial> referenceMaterialLambdaQueryWrapper = new LambdaQueryWrapper<>();

                referenceMaterialLambdaQueryWrapper.eq(ReferenceMaterial::getNumber,reagentConsumablesSet.getReferenceMaterialNumber());

                ReferenceMaterial referenceMaterial = referenceMaterialService.getOne(referenceMaterialLambdaQueryWrapper);

                outgoingContents.setReferenceMaterialId(referenceMaterial.getId());
            }

            outgoingContents.setId(IdWorker.get32UUID().toUpperCase());
            outgoingContents.setDeliveryRegistrationFormId(deliveryRegistrationForm.getId());
            outgoingContents.setQuantity(reagentConsumablesSet.getQuantity());
            outgoingContents.setRemarks(reagentConsumablesSet.getRemarks());
            outgoingContents.setOutboundUse(reagentConsumablesSet.getPurpose());
            outgoingContents.setReagentConsumableId(reagentConsumablesSet.getReagentConsumableId());
            outgoingContents.setReagentConsumableType(one.getCategory());
            outgoingContents.setReturnOrNot(reagentConsumablesSet.isReturnOrNot());
            outgoingContents.setReferenceMaterialNumber(reagentConsumablesSet.getReferenceMaterialNumber());

            outgoingContentsList.add(outgoingContents);

        }
        if (deliveryRegistrationFormService.save(deliveryRegistrationForm) && outgoingContentsService.saveBatch(outgoingContentsList) &
                applicationForUseService.updateById(byId)) {

            return deliveryRegistrationForm;
        } else throw new RuntimeException(String.format("创建出库任务失败"));
    }

    @Override//提交出库登记表
    @Transactional
    public DeliveryRegistrationForm commitForm(List<OutgoingContentsDTO> outgoingContentsDTOS, DLPUser dlpUser) {
        //得到出库登记表
        DeliveryRegistrationForm byId = deliveryRegistrationFormService.getById(outgoingContentsDTOS.get(0).getDeliveryRegistrationFormId());

        byId.setStatus(1);

        byId.setDepositorId(dlpUser.getId());

        LambdaQueryWrapper<OutgoingContents> outgoingContentsLambdaQueryWrapper = new LambdaQueryWrapper<>();

        outgoingContentsLambdaQueryWrapper.eq(OutgoingContents::getDeliveryRegistrationFormId, byId.getId());
        //删除修改之前的内容(防止领用为X种，但实际出库只有n种，导致领用出库不一致)
        outgoingContentsService.removeBatchByIds(outgoingContentsService.list(outgoingContentsLambdaQueryWrapper));

        List<StandardMaterialApplication> standardMaterialApplications = new ArrayList<>();

        List<OutgoingContents> outgoingContentsList = new ArrayList<>();

        LambdaQueryWrapper<ApplicationForUse> applicationForUseLambdaQueryWrapper = new LambdaQueryWrapper<>();

        applicationForUseLambdaQueryWrapper.eq(ApplicationForUse::getDeliveryRegistrationFormId, byId.getId());
        //查询领用申请表
        ApplicationForUse one = applicationForUseService.getOne(applicationForUseLambdaQueryWrapper);

        one.setClaimCode("0000");

        for (OutgoingContentsDTO outgoingContentsDTO : outgoingContentsDTOS) {

            OutgoingContents outgoingContents = new OutgoingContents();

            BeanUtils.copyProperties(outgoingContentsDTO, outgoingContents);

            outgoingContents.setId(IdWorker.get32UUID().toUpperCase());

            ReagentConsumables reagentConsumables = reagentConsumablesService.getById(outgoingContents.getReagentConsumableId());

            outgoingContents.setReagentConsumableType(reagentConsumables.getCategory());

            outgoingContents.setDeliveryRegistrationFormId(byId.getId());

            if (reagentConsumables.getCategory().equals("标准物质")&&outgoingContents.isReturnOrNot()) {
                //生成标准物质领用/归还登记表
                StandardMaterialApplication standardMaterialApplication = new StandardMaterialApplication();

                standardMaterialApplication.setStandardMaterialApplicationId(IdWorker.get32UUID().toUpperCase());
                standardMaterialApplication.setPurpose(outgoingContents.getOutboundUse());
                standardMaterialApplication.setDateOfClaim(LocalDateTime.now());
                standardMaterialApplication.setRecipientId(one.getRecipientId());
                standardMaterialApplication.setReferenceSubstanceId(reagentConsumables.getReagentConsumableId());
                standardMaterialApplication.setSpecification(reagentConsumables.getSpecificationAndModel());
                standardMaterialApplication.setRequisitionedQuantity(outgoingContents.getQuantity());
                standardMaterialApplication.setSpecification(reagentConsumables.getSpecificationAndModel());
                standardMaterialApplication.setApplicationForUseId(one.getId());
                standardMaterialApplication.setReferenceMaterialId(outgoingContentsDTO.getReferenceMaterialId());
                standardMaterialApplication.setStatus(0);
                standardMaterialApplication.setReferenceMaterialNumber(outgoingContents.getReferenceMaterialNumber());
                standardMaterialApplications.add(standardMaterialApplication);

                ReferenceMaterial byId1 = referenceMaterialService.getById(outgoingContentsDTO.getReferenceMaterialId());

                byId1.setStatus(-1);
                //查找出对应的仓库信息，将库存量减少
                ReagentConsumableInventory reagentConsumableInventory = reagentConsumableInventoryService.reduceById(outgoingContents.getReagentConsumableId(), outgoingContents.getQuantity());

                if (reagentConsumableInventory.getTotalQuantity() < 0) {

                    throw new RuntimeException(String.format("出库数量不能超过库存量"));
                }

            } else if (reagentConsumables.getCategory().equals("标准储备溶液")&&outgoingContents.isReturnOrNot()) {
                //创建标准物质领用/归还登记表
                StandardMaterialApplication standardMaterialApplication = new StandardMaterialApplication();
                standardMaterialApplication.setReferenceMaterialNumber(outgoingContents.getReferenceMaterialNumber());
                standardMaterialApplication.setStandardMaterialApplicationId(IdWorker.get32UUID().toUpperCase());
                standardMaterialApplication.setPurpose(outgoingContents.getOutboundUse());
                standardMaterialApplication.setDateOfClaim(LocalDateTime.now());
                standardMaterialApplication.setRecipientId(one.getRecipientId());
                standardMaterialApplication.setReferenceSubstanceId(reagentConsumables.getReagentConsumableId());
                standardMaterialApplication.setSpecification(reagentConsumables.getSpecificationAndModel());
                standardMaterialApplication.setRequisitionedQuantity(outgoingContents.getQuantity());
                standardMaterialApplication.setSpecification(reagentConsumables.getSpecificationAndModel());
                standardMaterialApplication.setApplicationForUseId(one.getId());
                standardMaterialApplication.setStatus(0);
                standardMaterialApplication.setReferenceMaterialId(outgoingContentsDTO.getReferenceMaterialId());
                standardMaterialApplications.add(standardMaterialApplication);

                solutionUseFormService.useById(outgoingContentsDTO.getReferenceMaterialId(), dlpUser);
                //通过标准物质ID，查找出对应的仓库信息，将库存量减少
                String referenceMaterialId = outgoingContentsDTO.getReferenceMaterialId();

                ReferenceMaterial referenceMaterial = referenceMaterialService.getById(referenceMaterialId);

                referenceMaterial.setStatus(-1);

                ReagentConsumableInventory reagentConsumableInventory = reagentConsumableInventoryService.getById(referenceMaterial.getReagentConsumableInventoryId());

                reagentConsumableInventory.setTotalQuantity(reagentConsumableInventory.getTotalQuantity() - outgoingContentsDTO.getQuantity());

                if (reagentConsumableInventory.getTotalQuantity() < 0) {
                    throw new RuntimeException(String.format("出库数量不能超过库存量"));
                }

            } else {
                RequisitionRecord requisitionRecord = requisitionRecordService.addRecord(outgoingContents);

                requisitionRecord.setDateOfClaim(LocalDateTime.now());
                requisitionRecord.setRecipientId(one.getRecipientId());

                requisitionRecordService.updateById(requisitionRecord);
                //查找出对应的仓库信息，将库存量减少
                ReagentConsumableInventory reagentConsumableInventory = reagentConsumableInventoryService.reduceById(outgoingContents.getReagentConsumableId(), outgoingContents.getQuantity());

                if (reagentConsumableInventory.getTotalQuantity() < 0) {
                    throw new RuntimeException(String.format("出库数量不能超过库存量"));
                }
            }
            //如果用途是用作案件，则需创建符合性检查记录表
            if (outgoingContents.getOutboundUse() == 0) {
                ComplianceCheckDTO complianceCheckDTO = new ComplianceCheckDTO();
                complianceCheckDTO.setBrand(reagentConsumables.getBrand());
                complianceCheckDTO.setReagentConsumableId(reagentConsumables.getReagentConsumableId());
                complianceCheckDTO.setSpecificationAndModel(reagentConsumables.getSpecificationAndModel());
                complianceCheckDTO.setComplianceCheckId(IdWorker.get32UUID().toUpperCase());
                complianceCheckDTO.setReagentConsumableNumber(outgoingContentsDTO.getNumber());
                complianceCheckService.addCheck(complianceCheckDTO, dlpUser);
            }
            outgoingContentsList.add(outgoingContents);
        }
        if (outgoingContentsService.saveBatch(outgoingContentsList) && standardMaterialApplicationService.saveBatch(standardMaterialApplications)) {
            return byId;
        } else throw new RuntimeException(String.format("提交失败"));
    }

    @Override//分页查询
    public Page <DeliveryRegistrationFormVO> getDeliveryRegistrationFormVOPage(Page page, QueryWrapper<DeliveryRegistrationForm> qw){

        Page deliveryRegistrationFormVOPage = baseMapper.getDeliveryRegistrationFormVOPage(page, qw);

        return deliveryRegistrationFormVOPage;
    }

    @Override//通过ID查询出库内容
    public DeliveryRegistrationFormVO getDeliveryRegistrationFormVOById(String deliveryRegistrationFormId){

        DeliveryRegistrationFormVO deliveryRegistrationFormVOById = baseMapper.getDeliveryRegistrationFormVOById(deliveryRegistrationFormId);

        LambdaQueryWrapper<OutgoingContents> outgoingContentsLambdaQueryWrapper = new LambdaQueryWrapper<>();

        outgoingContentsLambdaQueryWrapper.eq(OutgoingContents::getDeliveryRegistrationFormId,deliveryRegistrationFormVOById.getId());

        List<OutgoingContents> list = outgoingContentsService.list(outgoingContentsLambdaQueryWrapper);

        List<OutgoingContentsVO> outgoingContentsVOS = new ArrayList<>();

        if (list.size()!=0){

        for (OutgoingContents outgoingContents : list) {

            OutgoingContentsVO outgoingContentsVO = new OutgoingContentsVO();

            BeanUtils.copyProperties(outgoingContents,outgoingContentsVO);

            LambdaQueryWrapper<ReagentConsumableInventory> reagentConsumableInventoryLambdaQueryWrapper = new LambdaQueryWrapper<>();

            reagentConsumableInventoryLambdaQueryWrapper.eq(ReagentConsumableInventory::getReagentConsumableId,outgoingContents.getReagentConsumableId());

            ReagentConsumableInventory one = reagentConsumableInventoryService.getOne(reagentConsumableInventoryLambdaQueryWrapper);

            outgoingContentsVO.setReagentConsumableInventory(one);

            outgoingContentsVOS.add(outgoingContentsVO);
        }

        deliveryRegistrationFormVOById.setOutgoingContentsList(outgoingContentsVOS);}

        return deliveryRegistrationFormVOById;
    }

    @Override
    public ReagentConsumablesVO scanById(String reagentConsumableId, String referenceMaterialId){

        if (!(referenceMaterialId.equals("0"))){

            ReagentConsumables byId = reagentConsumablesService.getById(reagentConsumableId);

            ReferenceMaterial byId1 = referenceMaterialService.getById(referenceMaterialId);

            ReagentConsumablesVO reagentConsumablesVO = new ReagentConsumablesVO();

            BeanUtils.copyProperties(byId,reagentConsumablesVO);

            reagentConsumablesVO.setNumber(byId1.getNumber());

            reagentConsumablesVO.setReferenceMaterialId(referenceMaterialId);

            return reagentConsumablesVO;
        }else {

            ReagentConsumables byId = reagentConsumablesService.getById(reagentConsumableId);

            ReagentConsumablesVO reagentConsumablesVO = new ReagentConsumablesVO();

            BeanUtils.copyProperties(byId,reagentConsumablesVO);

            return reagentConsumablesVO;
        }
    }
}
